use warnings;
use strict;

package Net::Domain::SMD::File;
use parent 'Net::Domain::SMD';

use Log::Report   'net-domain-smd';

use MIME::Base64       qw/decode_base64/;
use XML::LibXML        ();
use POSIX              qw/mktime tzset/;
use XML::Compile::Util qw/type_of_node/;
use List::Util         qw/first/;

=chapter NAME
Net::Domain::SMD::File - Info from one SMD file

=chapter SYNOPSIS

  # end-user route to collect the info
  my $tmch  = Net::Domain::TMCH->new(%options);
  my $smdf  = $tmch->smd($filename);

  # used by TMCH internally via ::SMD
  my $smd  = Net::Domain::SMD::Schema->new(%options);
  my $smdf = $smd->read($fn);

  # used by ::SMD internally as
  my $smdf = Net::Domain::SMD->fromFile($fn, schemas => $smd->schemas);

  # See Net::Domain::SMD for generic interface

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-spec-06>
SMD information is packaged.  This object represents one file in wrapped
format.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=requires filename FILENAME
=cut

sub init($)
{   my ($self, $args) = @_;
    $self->SUPER::init($args);

    # Clean object construction is needed when we are going to write
    # SMD files... but we won't for now.
    $self->{NDSF_fn}     = $args->{filename} or panic;
    $self->{NDSF_marks}  = $args->{marks};
    $self->{NDSF_labels} = $args->{labels};
    $self;
}

=c_method fromFile FILENAME, OPTIONS

=requires schemas SCHEMAS

=cut

sub fromFile($%)
{   my ($class, $fn, %args) = @_;

    my $schemas = $args{schemas} or panic;

    open my($fh), '<:raw', $fn
        or fault __x"cannot read from smd file {fn}", fn => $fn;

    my $xml;
    my ($marks, $labels);
  LINE:
    while(<$fh>)
    {   next LINE if m/^#|^\s*$/;   # not yet permitted in those files
        if( m/^-{3,}BEGIN .* SMD/)
        {   my @smd;
            while(<$fh>)
            {   last if m/^-{3,}END .* SMD/;
                push @smd, $_;
            }
            $xml = \decode_base64(join '', @smd);
            next LINE;
        }

        # Only few of the fields are of interest: often better inside XML
        my ($label, $value) = split /\:\s+/;
        defined $value && length $value or next;
        $label = lc $label;
        $value =~ s/\s*$//s;
        if($label eq 'u-labels')
        {   $labels = [split /\s*\,\s*/, $value];
        }
        elsif($label eq 'marks')  # trademark names?  Comma list?
        {   $marks  =  [split /\s*\,\s*/, $value];
        }

    }

    $xml or error __x"there is not SMD information in {fn}", fn => $fn;

    my $root = $schemas->dataToXML($xml);
    $class->fromNode($root, filename => $fn, marks => $marks
      , labels => $labels, %args);
}
    
#----------------
=section Attributes

=method filename

=method labels
Returns a list of u-labels.

=method marks
Returns the marks named in the "Marks" field.  The name insinuates that
this field may contains multiple values, but the format of this field
is not described in the spec.

=cut

sub filename()  {shift->{NDSF_fn}}
sub labels()    { @{shift->{NDSF_labels} || []} }
sub marks()     { @{shift->{NDSF_marks}  || []} }

#----------------
=section File header access

The SMD-file contains a header, wrapping the XML content.  The next
methods provide access to that header information.

=method smdID

=cut

sub smdID()     {shift->data->{smd_id}}

1;
