use warnings;
use strict;

package Net::Domain::TMCH;
use base 'Exporter';

use Log::Report                  'net-domain-smd';

use Net::Domain::SMD::Schema   ();
use Net::Domain::TMCH::CRL     ();
use Net::Domain::SMD::RL       ();

use Crypt::OpenSSL::VerifyX509 ();
use Crypt::OpenSSL::X509   ();
use File::Basename         qw(dirname);
use File::Spec::Functions  qw(catfile);
use Scalar::Util           qw(blessed);
use URI                    ();

use constant
  { CRL_SOURCE => 'http://crl.icann.org/tmch.crl'   # what? no https?
  };

=chapter NAME
Net::Domain::TMCH - TradeMark Clearing House

=chapter SYNOPSIS

  my $tmch  = Net::Domain::TMCH->new(%options);

  my $smd   = $tmch->smd($filename);

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-07>
specifies a way for TLD (domain registries) to exchange information
about trademark rights, brand registrations.

This module manages the verification process of "SMD"-files, each
containing a crypto-signed XML structure of information related to
a single trademark claim.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=option  tmch_certificate FILENAME
=default tmch_certificate <included>
This must be a PEM file which contains the certificate on top of all
TMCH certificates.

=option  cert_revocations FILENAME|URI|OBJECT
=default cert_revocations <according to rfc>
The CRL for invalidated TMV certificates.  You may also provide an
M<Net::Domain::TMCH::CRL> object.

=option  is_pilot BOOLEAN
=default is_pilot <false>

=option  smd_revocations FILENAME|URI|ARRAY
=default smd_revocations []

=option  smd_admin M<Net::Domain::SMD::Schema>-object
=default smd_admin <generated internally>

=option  auto_datetime BOOLEAN
=default auto_datetime <false>
Convert dates in the XML automatically into M<DateTime> objects.

=cut

sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }

sub init($)
{   my ($self, $args) = @_;
    $self->{NDT_smds} = $args->{smds_admin} ||
        Net::Domain::SMD::Schema->new
         ( prepare       => 'READER'
         , auto_datetime => $args->{auto_datetime}
         );

    my $stage    = $self->{NDT_pilot}
      = $args->{is_pilot} ? 'tmch_pilot' : 'tmch';

    my $tmch_pem = $args->{tmch_certificate}
     || catfile dirname(__FILE__), 'TMCH', 'icann', "$stage.pem";

    $self->{NDT_tmch_cert} = Crypt::OpenSSL::X509->new_from_file($tmch_pem);
    $self->{NDT_tmch_ca}   = Crypt::OpenSSL::VerifyX509->new($tmch_pem);

    $self->{NDT_crl}   = $self->_crl($args->{cert_revocations} || CRL_SOURCE);
    $self->{NDT_smdrl} = [ $self->_smdrl($args->{smd_revocations}) ];

    $self;
}

sub _crl($)
{   my ($self, $r) = @_;

    $r = URI->new($r)
        if !blessed $r && $r =~ m!^https?://!;

    return Net::Domain::TMCH::CRL->fromFile($r)
        if !blessed $r;

    return $r
        if $r->isa('Net::Domain::TMCH::CRL');

    return Net::Domain::TMCH::CRL->fromURI($r)
        if $r->isa('URI');

    error __x"revocation_list for THMC is not a {pkg}, filename, or uri"
      , pkg => 'Net::Domain::TMCH::CRL';
}

sub _smdrl($)
{   my ($self, $r) = @_;

    return ()
        unless defined $r;

    return map $self->_smdrl($_), @$r
        if ref $r eq 'ARRAY';

    $r = URI->new($r)
        if !blessed $r && $r =~ m!^https?://!;

    return Net::Domain::SMD::RL->fromFile($r)
        if !blessed $r;

    return $r
        if $r->isa('Net::Domain::SMD::RL');

    return Net::Domain::SMD::RL->fromURI($r)
        if $r->isa('URI');
    
    error __x"revocation_list for SMD is not a {pkg} or filename"
      , pkg => 'Net::Domain::SMD::RL';
}

#-------------------------

=section Attributes
=method smdAdmin
=method isPilot
=method tmchCertficate
=method tmchCA
=method certRevocations
=method smdRevocations
=cut

sub smdAdmin()       {shift->{NDT_smds}}
sub isPilot()        {shift->{NDT_pilot}}
sub tmchCertificate(){shift->{NDT_tmch_cert}}
sub tmchCA()         {shift->{NDT_tmch_ca}}
sub certRevocations(){shift->{NDT_crl}}
sub smdRevocations() { @{shift->{NDT_smdrl}} }

#-------------------------

=section Actions

=method smd (FILENAME|XML|STRING), OPTIONS
The first argument can be either a FILENAME (in which case an
M<Net::Domain::SMD::File> object is returned, or an M<XML::LibXML::Element>
or a STRING (bother returning an M<Net::Domain::SMD> object).  In either
case, the returned object reflects what is kept inside the XML.

C<undef> is returned when the data is invalid, for instance because it
has expired.

=option  accept_expired BOOLEAN
=default accept_expired <false>
When true, you will be able to access expired SMD files.  This is
especially useful in regression tests, or when you process ancient
SMD files.

=option  trust_certificates BOOLEAN
=default trust_certificates <false>
Do not check the certificates inside the SMD file for validity.

=example
   my $smd = $tmch->smd($fh)
       or die "cannot use $fn: $@";

=cut

sub smd($%)
{   my ($self, $xml, %args) = @_;

    my ($smd, $source) = $self->smdAdmin->from($xml);
    return $smd
        if !$smd || $args{trust_certificates};

    my $tmch_cert = $self->tmchCertificate;

    my ($tmv_cert) = $smd->certificates(issuer => $tmch_cert->subject);
    defined $tmv_cert
        or error __x"smd in {source} does not contain an TMV certificate"
             , source => $source;

    $self->tmchCA->verify($tmv_cert)
        or error __x"invalid TMV certificate in {source}", source => $source;

    $args{accept_expired} || ! $tmv_cert->checkend(0)
        or error __x"the TMV certificate in {source} has expired"
             , source => $source;

    $self->certRevocations->isRevoked($tmv_cert)
        and error __x"smd in {source} contains revoked TMV certificate"
             , source => $source;

    foreach my $rl ($self->smdRevocations)
    {   error __x"smd in {source} is revoked according to {list}"
          , source => $source, list => $rl->source
            if $rl->isRevoked($smd);
    }

    $smd;
}

1;
